%% Example Interface Functions for TBV Network Interface.
%% Michael Luehrs, Maastricht University, Brain Innovation B.V., 08 April 2020


        % --------------------
        %% Configure IP and Port
        % --------------------

configs.TBV_IP = '127.0.0.1';
configs.TBV_PORT = 55555;


        % --------------------
        %% Create Connection
        % --------------------

tbvNetInt = TBVNetworkInterface( TBVclient( configs.TBV_IP, configs.TBV_PORT ) );
tbvNetInt.createConnection();


        % --------------------
        %% Basic TBV access functions
        % --------------------
        
timePoint = tbvNetInt.tGetCurrentTimePoint();
NrOfTimePoints = tbvNetInt.tGetExpectedNrOfTimePoints();
[dim_x, dim_y, dim_z] = tbvNetInt.tGetDimsOfFunctionalData();
projectName = tbvNetInt.tGetProjectName();
cWatchFolder = tbvNetInt.tGetWatchFolder();
cTargetFolder = tbvNetInt.tGetTargetFolder();
feedbackFolder = tbvNetInt.tGetFeedbackFolder();

     
        % --------------------
        %% Protocol, DM, GLM Queries
        % --------------------
        
CurrentProtocolCondition = tbvNetInt.tGetCurrentProtocolCondition();
fullNrOfPredictors = tbvNetInt.tGetFullNrOfPredictors();
currentNrOfPredictors = tbvNetInt.tGetCurrentNrOfPredictors();
nrOfConfoundPredictors = tbvNetInt.tGetNrOfConfoundPredictors();
[pred, timepoint, valueOfDesignMatrix] = tbvNetInt.tGetValueOfDesignMatrix( 2, timePoint-1);
nrOfContrasts = tbvNetInt.tGetNrOfContrasts();
nrOfROI = tbvNetInt.tGetNrOfROIs();
meanOfROI = tbvNetInt.tGetMeanOfROI( nrOfROI);
[meanOfROI, timePnt] = tbvNetInt.tGetMeanOfROIAtTimePoint( nrOfROI, timePoint-1);
existingmeanOfROI= tbvNetInt.tGetExistingMeansOfROI( 1, timePoint-1);


%% IMPORTANT %%
%% Detrending must be turned on in TBV to be applied for all functions below!!! Otherwise non detrended time courses will be used as input. %%
DetrendedmeanOfROI = tbvNetInt.tGetDetrendedValueOfROI(nrOfROI);
[detrendedmeanOfROI, timePnt] = tbvNetInt.tGetDetrendedMeanOfROIAtTimePoint( nrOfROI, timePoint-1);
existingdetrendedmeanOfROI = tbvNetInt.tGetExistingDetrendedMeansOfROI( 1, timePoint-1);


figure;
plot(existingmeanOfROI)
hold on;
plot(existingdetrendedmeanOfROI)


NrOfVoxelsOfROI = tbvNetInt.tGetNrOfVoxelsOfROI( nrOfROI);
betaOfROI = tbvNetInt.tGetBetaOfROI( nrOfROI, currentNrOfPredictors);
[x, y, z] = tbvNetInt.tGetCoordsOfVoxelOfROI( nrOfROI, NrOfVoxelsOfROI-1);
coordsOfVoxelsOfROI = tbvNetInt.tGetAllCoordsOfVoxelsOfROI( nrOfROI);


        % --------------------
        %% Volume Data Access Queries
        % --------------------
        
[ValueOfVoxelAtTime] = tbvNetInt.tGetValueOfVoxelAtTime( x, y, z, timePoint-1);
valueOfAllVoxelsAtTime = tbvNetInt.tGetValueOfAllVoxelsAtTime( timePoint-1);valueOfAllVoxelsAtTime = reshape(valueOfAllVoxelsAtTime,[dim_x,dim_y,dim_z]);
figure;  
squareP = ceil(sqrt(size(valueOfAllVoxelsAtTime,3)));
for i = 1:size(valueOfAllVoxelsAtTime,3)
    subs = subplot(squareP,squareP,i);
    imagesc(flip(rot90(valueOfAllVoxelsAtTime(:,:,i))))
    set(subs,'XTick',[], 'YTick', [])
end
RawValueTimeCourseData = tbvNetInt.tGetRawValueOfAllVoxelsAtTime( timePoint-1);RawValueTimeCourseData = reshape(RawValueTimeCourseData,[dim_x,dim_y,dim_z]);
figure;  

for i = 1:size(RawValueTimeCourseData,3)
    subs = subplot(squareP,squareP,i);
    imagesc(flip(rot90(RawValueTimeCourseData(:,:,i))))
    set(subs,'XTick',[], 'YTick', [])
end
BetaOfVoxel = tbvNetInt.tGetBetaOfVoxel( currentNrOfPredictors, x, y, z);
BetaMaps = tbvNetInt.tGetBetaMaps();
if length(BetaMaps) > 0
    figure;
    betaMapsR = reshape(BetaMaps(:),[dim_x,dim_y,dim_z,nrOfContrasts+1]);
    squareP = ceil(sqrt(size(betaMapsR,3)));
    for i = 1:size(betaMapsR,3)
        subs = subplot(squareP,squareP,i);
        imagesc(flip(rot90(betaMapsR(:,:,i,1))))
        set(subs,'XTick',[], 'YTick', [])
    end
end
MapValueOfVoxel = tbvNetInt.tGetMapValueOfVoxel ( 1, x, y, z);
contrastMaps = tbvNetInt.tGetContrastMaps();
if length(contrastMaps) > 0
    figure;
    contrastMapsR = reshape(contrastMaps(:),[dim_x,dim_y,dim_z,nrOfContrasts]);
    squareP = ceil(sqrt(size(contrastMapsR,3)));
    for i = 1:size(contrastMapsR,3)
        subs = subplot(squareP,squareP,i);
        imagesc(flip(rot90(contrastMapsR(:,:,i,1))))
        set(subs,'XTick',[], 'YTick', [])
        
    end
end


        % --------------------
        %% SVM Functions  
        % --------------------
       
numClasses = tbvNetInt.tGetNumberOfClasses();
if numClasses > -1
    [winnerclass, winnerval] = tbvNetInt.tGetCurrentClassifierOutput();
end

        % --------------------
        %% Functional Connectivity Functions  
        % --------------------
        
[wSize,PearsonCorrelation] = tbvNetInt.tGetPearsonCorrelation( 20);
[wSize,tPoint,PearsonCorrelationAtTimePoint] = tbvNetInt.tGetPearsonCorrelationAtTimePoint( 20, timePoint);
[wSize,PartialCorrelation] = tbvNetInt.tGetPartialCorrelation( 20);
[wSize,tPoint,PartialCorrelationAtTimePoint] = tbvNetInt.tGetPartialCorrelationAtTimePoint( 20, timePoint);

[wSize,InstantProxyCorrelation] = tbvNetInt.tGetInstantProxyCorrelation( 20);
[wSize,tPoint,InstantProxyCorrelationAtTimePoint] = tbvNetInt.tGetInstantProxyCorrelationAtTimePoint( 20, timePoint);


%% IMPORTANT %%
%% Detrending must be turned on in TBV to be applied for all functions below!!! Otherwise non detrended time courses will be used as input. %%
[wSize,DetrendedPearsonCorrelation] = tbvNetInt.tGetDetrendedPearsonCorrelation( 20);
[wSize,tPoint,DetrendedPearsonCorrelationAtTimePoint] = tbvNetInt.tGetDetrendedPearsonCorrelationAtTimePoint( 20, timePoint);
[wSize,DetrendedPartialCorrelation] = tbvNetInt.tGetDetrendedPartialCorrelation( 20);
[wSize,tPoint,DetrendedPartialCorrelationAtTimePoint] = tbvNetInt.tGetDetrendedPartialCorrelationAtTimePoint( 20, timePoint);

[wSize,DetrendedInstantProxyCorrelation] = tbvNetInt.tGetDetrendedInstantProxyCorrelation( 20);
[wSize,tPoint,DetrendedInstantProxyCorrelationAtTimePoint] = tbvNetInt.tGetDetrendedInstantProxyCorrelationAtTimePoint( 20, timePoint);

